from typing import Optional

from climoji.domain.model import EmojiRepository, Emoji
from climoji.infrastructure._file_repository._emoji_list import EMOJI_ALIAS_UNICODE


class FileEmojiRepository(EmojiRepository):
    def find(self, alias: str) -> Optional[Emoji]:
        try:
            data = EMOJI_ALIAS_UNICODE[alias]
            return Emoji(alias, data)
        except KeyError:
            return None



from abc import ABCMeta, abstractmethod
from typing import Optional


class Emoji:
    def __init__(self, alias, emoji):
        self.emoji = emoji
        self.alias = alias


class EmojiRepository(metaclass=ABCMeta):
    @abstractmethod
    def find(self, alias: str) -> Optional[Emoji]:
        pass



import click

from climoji.domain.exception import DomainException
from climoji.infrastructure import container


@click.group()
def cli():
    pass


@cli.command()
@click.argument('alias')
def show(alias):
    try:
        emoji = container.finder().get(alias)
        click.echo(emoji.emoji)
    except DomainException as e:
        click.echo(e.message())



from abc import ABCMeta, abstractmethod


class DomainException(Exception, metaclass=ABCMeta):
    @abstractmethod
    def message(self) -> str:
        pass


class EmojiNotFoundException(DomainException):
    def __init__(self, alias: str):
        self.alias = alias

    def message(self) -> str:
        return 'Emoji "{}" not found'.format(self.alias)



from setuptools import setup, find_packages

setup(
    name="climoji",
    version="1.0a",
    packages=find_packages(),
    author="Petru Rares Sincraian",
    author_email="psincraian@gmail.com",
    description="A cli emoji finder",
    keywords="emoji cli finder",
    url="https://github.com/psincraian/climoji",
    license="MIT",
    entry_points={
        'console_scripts': [
            'climoji = climoji.infrastructure:cli'
        ]
    },
    install_requires=[
        'click>=6.7,<7'
    ]
)




def file_emoji_repository():
    from climoji.infrastructure import FileEmojiRepository
    return FileEmojiRepository()


def finder():
    from climoji.application import Finder
    return Finder(file_emoji_repository())



from climoji.domain.exception import EmojiNotFoundException
from climoji.domain.model import EmojiRepository, Emoji


class Finder:
    def __init__(self, emoji_projection: EmojiRepository):
        self._emoji_projection = emoji_projection

    def get(self, alias: str) -> Emoji:
        emoji = self._emoji_projection.find(alias)
        if emoji is None:
            raise EmojiNotFoundException(alias)
        return emoji
